@@ -1,15 +1,16 @@
-import collections
+from euphorie.client import model
+from euphorie.client.utils import HasText
+from euphorie.content.interfaces import IQuestionContainer
 from five import grok
 from z3c.saconfig import Session
-from euphorie.client import model
-from euphorie.content.interfaces import IQuestionContainer
-from euphorie.client.session import SessionManager
-from euphorie.client.utils import HasText
+
+import collections
+
 
 grok.templatedir("templates")
 
 
-def getSurveyTree(survey):
+def getSurveyTree(survey, profile=None):
     """Return a list of all modules, profile questions and risks in
     a survey. Each entry is represented by a dict with a `zodb_path`
     and `type` key.
@@ -24,10 +25,24 @@ def getSurveyTree(survey):
                 ['euphorie.profilequestion', 'euphorie.module',
                  'euphorie.risk']:
             continue
+        # Note that in profile.AddToTree, we pretend that an optional module
+        # always has a description. This logic needs to be replicated here.
+        if node.portal_type == 'euphorie.module':
+            has_description = HasText(node.description) or node.optional
+        else:
+            has_description = HasText(node.description)
+        if profile and node.portal_type == 'euphorie.profilequestion':
+            if not profile.get(node.id):
+                continue
         nodes.append({
             'zodb_path': '/'.join(node.getPhysicalPath()[base_length:]),
             'type': node.portal_type[9:],
-            'has_description': HasText(node.description),
+            'has_description': has_description,
+            'always_present': (
+                node.portal_type[9:] == "risk" and node.risk_always_present or
+                False),
+            'risk_type': (
+                node.portal_type[9:] == "risk" and node.type or None),
             'optional': node.optional, })
         if IQuestionContainer.providedBy(node):
             queue.extend(node.values())
@@ -45,6 +60,8 @@ class Node(object):
         self.type = item.type
         self.skip_children = item.skip_children
         self.has_description = item.has_description
+        self.identification = item.type == 'risk' and item.identification or None
+        self.risk_type = item.type == 'risk' and item.risk_type or None
 
     def __repr__(self):
         return "<Node zodb_path=%s type=%s path=%s>" % \
@@ -64,13 +81,13 @@ def getSessionTree(session):
     return [Node(item) for item in query]
 
 
-def treeChanges(session, survey):
+def treeChanges(session, survey, profile=None):
     """Determine a list of changes in a survey for an existing session. The
     list of changes is returned as a list of tuples listing the ZODB path,
     the object type (one of `profile`, `module` or `risk`) and the change type
     (one of `add`, `remove` or 'modified').
     """
-    surveytree = getSurveyTree(survey)
+    surveytree = getSurveyTree(survey, profile)
     sestree = set(getSessionTree(session))
     results = set()
     for entry in surveytree:
@@ -88,18 +105,31 @@ def treeChanges(session, survey):
                     results.add(
                             (entry["zodb_path"], nodes[0].type, "modified"))
                 elif entry['has_description'] != nodes[0].has_description:
+                    # Log module description changes since this changes
+                    # client behaviour: modules without description are
+                    # skipped.
                     results.add(
                             (entry["zodb_path"], nodes[0].type, "modified"))
+            if node.type == entry['type'] == 'risk':
+                if entry['always_present'] and node.identification != u'no':
+                    results.add(
+                        (entry["zodb_path"], node.type, "modified"))
+                if entry['risk_type'] != node.risk_type:
+                    results.add(
+                        (entry["zodb_path"], node.type, "modified"))
             if nodes[0].type == entry["type"] or \
                     (nodes[0].type == "module" and
                      entry["type"] == "profilequestion"):
                 continue
+            # Flag a type change as remove & add
             results.add((entry["zodb_path"], nodes[0].type, "remove"))
             results.add((entry["zodb_path"], entry["type"], "add"))
         else:
             results.add((entry["zodb_path"], entry["type"], "add"))
 
     for node in sestree:
+        if node.zodb_path.find('custom-risks') != -1:
+            continue
         results.add((node.zodb_path, node.type, "remove"))
 
     return results
@@ -121,26 +151,12 @@ def wasSurveyUpdated(session, survey):
         else:
             # BBB: Euphorie 1.x did not use a tuple to store extra information.
             timestamp = published
-        if session.modified >= timestamp:
+        if session.refreshed >= timestamp:
             return False
 
     changes = treeChanges(session, survey)
     if not changes:
-        session.touch()
+        session.refresh_survey()
         return False
 
     return True
-
-
-def redirectOnSurveyUpdate(request):
-    """Utility method for views to check if a survey has been updated,
-    and if so redirect the user to the update confirmation page is
-    generated. The return value is `True` if an update is required and
-    `False` otherwise."""
-    survey = request.survey
-    dbsession = SessionManager.session
-    if not wasSurveyUpdated(dbsession, survey):
-        return False
-
-    request.response.redirect("%s/update" % survey.absolute_url())
-    return True
