@@ -7,39 +7,48 @@ structure containing modules, profile questions and questions. Both modules and
 profile questions are used to create the hierarchy.
 """
 
-import sys
-from Acquisition import aq_inner
-from Acquisition import aq_parent
-from ZODB.POSException import ConflictError
-from OFS.event import ObjectClonedEvent
-from zope.interface import implements
-from zope.component import adapts
-from zope.component import getMultiAdapter
-from zope.event import notify
-from zope import schema
-from five import grok
-from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
-from zope.container.interfaces import INameChooser
-from plone.directives import form
-from plone.directives import dexterity
-from plone.app.z3cform.wysiwyg import WysiwygFieldWidget
-from plone.app.dexterity.behaviors.metadata import IBasic
-from plonetheme.nuplone.skin.interfaces import NuPloneSkin
-from plonetheme.nuplone.skin import actions
-from plone.indexer import indexer
-from htmllaundry.z3cform import HtmlText
-from Products.statusmessages.interfaces import IStatusMessage
-from Products.Archetypes.utils import shasattr
-from .behaviour.uniqueid import INameFromUniqueId
+from .. import MessageFactory as _
 from .behaviour.uniqueid import get_next_id
-from .profilequestion import IProfileQuestion
-from .module import IModule
+from .behaviour.uniqueid import INameFromUniqueId
+from .datamanager import ParentAttributeField
 from .fti import check_fti_paste_allowed
 from .interfaces import IQuestionContainer
 from .interfaces import ISurveyUnpublishEvent
+from .module import IModule
+from .profilequestion import IProfileQuestion
+from .utils import DragDropHelper
 from .utils import StripMarkup
-from .datamanager import ParentAttributeField
-from .. import MessageFactory as _
+from Acquisition import aq_base
+from Acquisition import aq_chain
+from Acquisition import aq_inner
+from Acquisition import aq_parent
+from euphorie.content.dependency import ConditionalHtmlText
+from euphorie.content.dependency import ConditionalTextLine
+from euphorie.content.utils import get_tool_type_default
+from euphorie.content.utils import IToolTypesInfo
+from five import grok
+from htmllaundry.z3cform import HtmlText
+from OFS.event import ObjectClonedEvent
+from plone.app.dexterity.behaviors.metadata import IBasic
+from plone.app.z3cform.wysiwyg import WysiwygFieldWidget
+from plone.directives import dexterity
+from plone.directives import form
+from plone.indexer import indexer
+from plonetheme.nuplone.skin import actions
+from plonetheme.nuplone.skin.interfaces import NuPloneSkin
+from plonetheme.nuplone.z3cform.directives import depends
+from Products.CMFCore.utils import getToolByName
+from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
+from Products.statusmessages.interfaces import IStatusMessage
+from ZODB.POSException import ConflictError
+from zope import schema
+from zope.component import getMultiAdapter
+from zope.component import getUtility
+from zope.container.interfaces import INameChooser
+from zope.event import notify
+from zope.interface import implements
+
+import sys
 
 
 grok.templatedir("templates")
@@ -53,7 +62,7 @@ class ISurvey(form.Schema, IBasic):
     title = schema.TextLine(
             title=_("label_survey_title", default=u"Version name"),
             description=_("help_survey_title",
-                default=u"This is the title of this survey version. This "
+                default=u"This is the title of this OiRA Tool version. This "
                         u"name is never shown to users."),
             required=True)
     form.order_before(title="*")
@@ -62,13 +71,11 @@ class ISurvey(form.Schema, IBasic):
 
     introduction = HtmlText(
             title=_("label_introduction", default=u"Introduction text"),
-            description=_("help_introduction",
-                default=u"The introduction text is shown when starting a "
-                        u"new survey session. If no introduction is "
-                        u"provided here a standard text will be shown."
-                        u"Please keep this text brief so it will easily "
-                        u"fit on screens of small devices such as "
-                        u"phones and PDAs"),
+            description=_(u"The introduction text is shown when starting a new "
+                    u"OiRA Tool session. If no introduction is provided here a "
+                    u"standard text will be shown. Please keep this text brief "
+                    u"so it will easily fit on screens of small devices such as "
+                    u"phones and PDAs."),
             required=False)
     form.widget(introduction=WysiwygFieldWidget)
 
@@ -76,7 +83,7 @@ class ISurvey(form.Schema, IBasic):
             title=_("label_evaluation_optional",
                 default=u"Evaluation may be skipped"),
             description=_("help_evaluation_optional",
-                default=u"This option allows uses to skip the evaluation "
+                default=u"This option allows users to skip the evaluation "
                         u"phase."),
             default=False,
             required=False)
@@ -87,7 +94,7 @@ class ISurvey(form.Schema, IBasic):
             default=u"en",
             required=True)
 
-    classification_code = schema.ASCIILine(
+    classification_code = schema.TextLine(
             title=_("label_classification_code",
                 default=u"Classification code"),
             description=_("help_classification_code",
@@ -96,6 +103,44 @@ class ISurvey(form.Schema, IBasic):
                         u"and based on revision 2 of the NACE standard."),
             required=False)
 
+    tool_type = schema.Choice(
+        title=_(
+            "label_tool_type",
+            default=u"Type of OiRA Tool"),
+        description=_(
+            "description_tool_type",
+            default=u'This selection determines, which variant of an OiRA Tool'
+            u' will be created. If you are not sure, select "Classic".'),
+        vocabulary="euphorie.tool_types_vocabulary",
+        defaultFactory=get_tool_type_default,
+        required=True)
+
+    enable_tool_notification = schema.Bool(
+        title=_("label_enable_tool_notification",
+                default=u"Show a custom notification for this OiRA tool?"),
+        description=_(
+            u'description_tool_notification',
+            default=u'If you enter text here, it will be shown to users '
+            u'in a pop-up when they open the tool. It can be used for '
+            u'notifying users about changes.'),
+        required=False,
+        default=False)
+
+    depends("tool_notification_title",
+            "enable_tool_notification",
+            "on")
+    tool_notification_title = ConditionalTextLine(
+        title=_("label_tool_notification_title", default=u"Tool notification title"),
+        required=True)
+
+    depends("tool_notification_message",
+            "enable_tool_notification",
+            "on")
+    tool_notification_message = ConditionalHtmlText(
+        title=_(
+            "label_tool_notification", default=u"Tool notification message"),
+        required=True)
+    form.widget(tool_notification_message=WysiwygFieldWidget)
 
 
 class SurveyAttributeField(ParentAttributeField):
@@ -105,6 +150,18 @@ class SurveyAttributeField(ParentAttributeField):
     }
 
 
+def get_tool_type(context):
+    """Return the tool type used in a given context. The type is set on
+    the survey.
+    """
+    from euphorie.content.survey import ISurvey  # XXX Circular
+    tt_default = get_tool_type_default()
+    for parent in aq_chain(aq_inner(context)):
+        if ISurvey.providedBy(parent):
+            return getattr(parent, 'tool_type', '') or tt_default
+    return tt_default
+
+
 class Survey(dexterity.Container):
     """A risk assessment survey.
 
@@ -119,7 +176,7 @@ class Survey(dexterity.Container):
 
     def _canCopy(self, op=0):
         """Tell Zope2 that this object can not be copied."""
-        return False
+        return op
 
     def _get_id(self, orig_id):
         """Pick an id for pasted content."""
@@ -134,16 +191,6 @@ class Survey(dexterity.Container):
         if validate_src:
             check_fti_paste_allowed(self, object)
 
-    @property
-    def surveygroup_title(self):
-        import pdb ; pdb.set_trace()
-        return u'Oops'
-
-    @surveygroup_title.setter
-    def surveygroup_title(self, value):
-        import pdb ; pdb.set_trace
-        print value
-
     def hasProfile(self):
         """Check if this survey has any profile questions.
 
@@ -157,11 +204,26 @@ class Survey(dexterity.Container):
         else:
             return False
 
+    def hasNotification(self):
+        """
+            Checks if a notification message was set
+        """
+        return self.enable_tool_notification or False
+
     def ProfileQuestions(self):
         """Return a list of all profile questions."""
         return [child for child in self.values()
                 if IProfileQuestion.providedBy(child)]
 
+    def get_tool_type_name(self):
+        """ Returns the human readable name of the chosen tool type """
+        my_tool_type = get_tool_type(self)
+        tti = getUtility(IToolTypesInfo)
+        tool_types = tti()
+        if my_tool_type not in tool_types:
+            my_tool_type = tti.default_tool_type
+        return tool_types[my_tool_type]["title"]
+
 
 @indexer(ISurvey)
 def SearchableTextIndexer(obj):
@@ -171,7 +233,7 @@ def SearchableTextIndexer(obj):
                      obj.classification_code or u""])
 
 
-class View(grok.View):
+class View(grok.View, DragDropHelper):
     grok.context(ISurvey)
     grok.require("zope2.View")
     grok.layer(NuPloneSkin)
@@ -198,7 +260,7 @@ class ISurveyAddSchema(form.Schema):
     title = schema.TextLine(
             title=_("label_survey_title", default=u"Version name"),
             description=_("help_survey_title",
-                default=u"This is the title of this survey version. This "
+                default=u"This is the title of this OiRA Tool version. This "
                         u"name is never shown to users."),
             required=True)
 
@@ -207,17 +269,17 @@ class ISurveyEditSchema(ISurvey):
     survey_title = schema.TextLine(
             title=_("label_title", default=u"Title"),
             description=_("help_surveygroup_title",
-                default=u"The title of this survey. This title is used in "
-                        u"the survey overview in the clients."),
+                default=u"The title of this OiRA Tool. This title is used in "
+                        u"the OiRA Tool overview in the clients."),
             required=True)
     form.order_before(survey_title="*")
 
     obsolete = schema.Bool(
             title=_("label_survey_obsolete",
-                default=u"Obsolete survey"),
+                default=u"Obsolete OiRA tool"),
             description=_("help_survey_obsolete",
-                default=u"This survey is obsolete; it has been retired or "
-                        u"replaced with another survey."),
+                default=u"This OiRA Tool is obsolete; it has been retired or "
+                        u"replaced with another OiRA Tool."),
             default=False,
             required=False)
     form.order_before(obsolete="introduction")
@@ -279,6 +341,14 @@ class Edit(form.SchemaEditForm):
 
     schema = ISurveyEditSchema
 
+    def applyChanges(self, data):
+        changes = super(Edit, self).applyChanges(data)
+        if changes:
+            # Reindex our parents title.
+            catalog = getToolByName(self.context, 'portal_catalog')
+            catalog.indexObject(aq_parent(aq_inner(self.context)))
+        return changes
+
 
 class Delete(actions.Delete):
     """Special delete action class which prevents deletion of published surveys
@@ -289,10 +359,10 @@ class Delete(actions.Delete):
     def verify(self, container, context):
         flash = IStatusMessage(self.request).addStatusMessage
 
-        if shasattr(container, 'published') and \
+        if hasattr(aq_base(container), 'published') and \
                 container.published == context.id:
             flash(_("message_no_delete_published_survey",
-                    default=u"You cannot delete a survey that is published. "
+                    default=u"You cannot delete an OiRA Tool version that is published. "
                             u"Please unpublish it first."),
                     "error")
             self.request.response.redirect(context.absolute_url())
@@ -307,7 +377,9 @@ class Delete(actions.Delete):
             return True
         else:
             flash(_("message_delete_no_last_survey",
-                    default=u"You can not delete the only survey version."),
+                    default=u"This is the only version of the OiRA Tool and can "
+                        u"therefore not be deleted. Did you perhaps want to "
+                        u"remove the OiRA Tool itself?"),
                     "error")
             self.request.response.redirect(context.absolute_url())
             return False
@@ -316,7 +388,7 @@ class Delete(actions.Delete):
 @grok.subscribe(ISurvey, ISurveyUnpublishEvent)
 def handleSurveyUnpublish(survey, event):
     """Event handler (subscriber) for unpublishing a survey."""
-    if shasattr(survey, "published"):
+    if hasattr(aq_base(survey), "published"):
         delattr(survey, "published")
 
     surveygroup = aq_parent(survey)
