@@ -1,7 +1,10 @@
-from euphorie.deployment.tests.functional import EuphorieTestCase
+# coding=utf-8
+from Acquisition import aq_parent
+from euphorie.testing import EuphorieIntegrationTestCase
 
 
-class ProfileQuestionTests(EuphorieTestCase):
+class ProfileQuestionTests(EuphorieIntegrationTestCase):
+
     def _create(self, container, *args, **kwargs):
         newid = container.invokeFactory(*args, **kwargs)
         return getattr(container, newid)
@@ -11,8 +14,9 @@ class ProfileQuestionTests(EuphorieTestCase):
         sector = self._create(country, "euphorie.sector", "sector")
         surveygroup = self._create(sector, "euphorie.surveygroup", "group")
         survey = self._create(surveygroup, "euphorie.survey", "survey")
-        pq = self._create(survey,
-                "euphorie.profilequestion", "profilequestion")
+        pq = self._create(
+            survey, "euphorie.profilequestion", "profilequestion"
+        )
         return pq
 
     def testNotGloballyAllowed(self):
@@ -24,10 +28,25 @@ class ProfileQuestionTests(EuphorieTestCase):
         self.loginAsPortalOwner()
         pq = self.createProfileQuestion()
         types = [fti.id for fti in pq.allowedContentTypes()]
-        self.assertEqual(set(types), set(["euphorie.module",
-                                          "euphorie.risk"]))
+        self.assertEqual(set(types), set(["euphorie.module", "euphorie.risk"]))
 
     def testCanBeCopied(self):
         self.loginAsPortalOwner()
         pq = self.createProfileQuestion()
         self.assertTrue(pq.cb_isCopyable())
+
+    def test_verifyObjectPaste_acceptablePaste(self):
+        self.loginAsPortalOwner()
+        target = self.createProfileQuestion()
+        survey = aq_parent(target)
+        source = self._create(survey, 'euphorie.module', 'other')
+        target._verifyObjectPaste(source)
+
+    def test_verifyObjectPaste_block_if_result_too_deep(self):
+        self.loginAsPortalOwner()
+        target = self.createProfileQuestion()
+        survey = aq_parent(target)
+        source = self._create(survey, 'euphorie.module', 'other')
+        other = self._create(source, 'euphorie.module', 'other')
+        self._create(other, 'euphorie.module', 'other')
+        self.assertRaises(ValueError, target._verifyObjectPaste, source)
