@@ -1,30 +1,52 @@
-import mimetypes
-import random
+"""
+Upload
+------
+
+Form and browser view for importing a previously exported survey in XML format.
+
+view: @@upload
+"""
+
+from .. import MessageFactory as _
+from .country import ICountry
+from .risk import EnsureInterface
+from .risk import IFrenchEvaluation
+from .risk import IKinneyEvaluation
+from .risk import IRisk
+from .sector import ISector
+from .user import LoginField
+from .user import validLoginValue
 from Acquisition import aq_inner
-import lxml.etree
-import lxml.objectify
+from euphorie.content.behaviors.toolcategory import IToolCategory
+from euphorie.content.utils import IToolTypesInfo
+from five import grok
+from plone.dexterity.utils import createContentInContainer
+from plone.directives import form
+from plone.namedfile import field as filefield
+from plone.namedfile.file import NamedBlobImage
+from Products.statusmessages.interfaces import IStatusMessage
+from z3c.appconfig.interfaces import IAppConfig
+from z3c.form import button
+from z3c.form.interfaces import WidgetActionExecutionError
 from zope import schema
+from zope.component import getMultiAdapter
+from zope.component import getUtility
 from zope.interface import Interface
 from zope.interface import Invalid
-from plone.namedfile.file import NamedBlobImage
-from .. import MessageFactory as _
-from plone.namedfile import field as filefield
-from z3c.form.interfaces import WidgetActionExecutionError
-from z3c.form import form, field, button
-from plone.z3cform.layout import wrap_form
-from plone.dexterity.utils import createContentInContainer
-from zope.component import getMultiAdapter
-from Products.statusmessages.interfaces import IStatusMessage
-from euphorie.content.risk import IRisk
-from euphorie.content.risk import IFrenchEvaluation
-from euphorie.content.risk import IKinneyEvaluation
-from euphorie.content.risk import EnsureInterface
-from euphorie.content.user import LoginField
-from euphorie.content.user import validLoginValue
+import lxml.etree
+import lxml.objectify
+import mimetypes
+import random
 
 
+ProfileQuestionLocationFields = [
+    'label_multiple_present',
+    'label_single_occurance',
+    'label_multiple_occurances']
+
 NSMAP = {None: "http://xml.simplon.biz/euphorie/survey/1.0"}
 XMLNS = "{%s}" % NSMAP[None]
+COMMA_REPLACEMENT = "__COMMA__"
 
 
 def attr_unicode(node, attr, default=None):
@@ -66,6 +88,8 @@ def el_bool(node, tag, default=False):
 
 
 class IImportSector(Interface):
+    """ The fields used for importing a :obj:`euphorie.content.sector`.
+    """
     sector_title = schema.TextLine(
             title=_("label_sector_title", default=u"Title of sector."),
             description=_("help_sector_title",
@@ -80,7 +104,7 @@ class IImportSector(Interface):
 
     surveygroup_title = schema.TextLine(
             title=_("label_surveygroup_title",
-                default=u"Title of imported survey."),
+                default=u"Title of imported OiRA Tool"),
             description=_("help_upload_surveygroup_title",
                 default=u"If you do not specify a title it will be taken "
                         u"from the input."),
@@ -88,7 +112,7 @@ class IImportSector(Interface):
 
     survey_title = schema.TextLine(
             title=_("label_upload_survey_title",
-                default=u"Name for survey version"),
+                default=u"Name for OiRA Tool version"),
             default=_(u"Standard"),
             required=True)
 
@@ -98,9 +122,11 @@ class IImportSector(Interface):
 
 
 class IImportSurvey(Interface):
+    """ The fields used for importing a :obj:`euphorie.content.survey`.
+    """
     surveygroup_title = schema.TextLine(
             title=_("label_surveygroup_title",
-                default=u"Title of imported survey."),
+                default=u"Title of imported OiRA Tool"),
             description=_("help_upload_surveygroup_title",
                 default=u"If you do not specify a title it will be taken "
                         u"from the input."),
@@ -108,8 +134,8 @@ class IImportSurvey(Interface):
 
     survey_title = schema.TextLine(
             title=_("label_upload_survey_title",
-                default=u"Name for survey version"),
-            default=_(u"Survey import"),
+                default=u"Name for OiRA Tool version"),
+            default=_(u"OiRA Tool import"),
             required=True)
 
     file = filefield.NamedFile(
@@ -118,16 +144,19 @@ class IImportSurvey(Interface):
 
 
 class SurveyImporter(object):
-    """Import a survey version from a XML file and creates a new survey group
+    """Import a survey version from an XML file and create a new survey group
     and survey. This assumes the current context is a sector.
     """
 
     def __init__(self, context):
         self.context = context
+        appconfig = getUtility(IAppConfig)
+        settings = appconfig.get('euphorie')
+        self.use_existing_measures = settings.get('use_existing_measures', False)
 
     def ImportImage(self, node):
         """
-        Import an image from an XML node.
+        Import a base64 encoded image from an XML node.
 
         :param node: lxml.objectified XML node of image element
         :rtype: (:py:class:`NamedImage`, unicode) tuple
@@ -147,14 +176,29 @@ class SurveyImporter(object):
         return (image, attr_unicode(node, "caption"))
 
     def ImportSolution(self, node, risk):
+        """
+        Create a new :obj:`euphorie.content.solution` object for a
+        :obj:`euphorie.content.risk` given the details for a Solution as an XML
+        node.
+
+        :returns: :obj:`euphorie.content.solution`.
+        """
         solution = createContentInContainer(risk, "euphorie.solution")
         solution.description = unicode(node.description)
         solution.action_plan = unicode(getattr(node, "action-plan"))
         solution.prevention_plan = el_unicode(node, "prevention-plan")
         solution.requirements = el_unicode(node, "requirements")
         solution.external_id = attr_unicode(node, "external-id")
+        return solution
 
     def ImportRisk(self, node, module):
+        """
+        Create a new :obj:`euphorie.content.risk` object for a
+        :obj:`euphorie.content.module` given the details for a Risk as an XML
+        node.
+
+        :returns: :obj:`euphorie.content.risk`.
+        """
         risk = createContentInContainer(module, "euphorie.risk",
                                       title=unicode(node.title))
         EnsureInterface(risk)
@@ -164,6 +208,10 @@ class SurveyImporter(object):
         risk.legal_reference = el_unicode(node, "legal-reference")
         risk.show_notapplicable = el_bool(node, "show-not-applicable")
         risk.external_id = attr_unicode(node, "external-id")
+        if self.use_existing_measures:
+            risk.existing_measures = \
+                el_unicode(node, "existing_measures")
+
         if risk.type == "risk":
             em = getattr(node, "evaluation-method")
             risk.evaluation_method = em.text
@@ -202,8 +250,16 @@ class SurveyImporter(object):
         if hasattr(node, "solutions"):
             for child in node.solutions.solution:
                 self.ImportSolution(child, risk)
+        return risk
 
     def ImportModule(self, node, survey):
+        """
+        Create a new :obj:`euphorie.content.module` object for a
+        :obj:`euphorie.content.survey` given the details for a Module as an XML
+        node.
+
+        :returns: :obj:`euphorie.content.module`.
+        """
         module = createContentInContainer(survey, "euphorie.module",
                                         title=unicode(node.title))
         module.optional = node.get("optional") == "true"
@@ -224,35 +280,67 @@ class SurveyImporter(object):
             (image, caption) = self.ImportImage(image)
             module.image = image
             module.caption = caption
+        return module
 
     def ImportProfileQuestion(self, node, survey):
-        profile = createContentInContainer(survey, "euphorie.profilequestion",
-                                        title=unicode(node.title))
-        profile.type = node.get("type")
+        """
+        Create a new :obj:`euphorie.content.profilequestion` object for a
+        :obj:`euphorie.content.survey` given the details for a Profile Question
+        as an XML node.
+
+        :returns: :obj:`euphorie.content.profilequestion`.
+        """
+        type = node.get("type")
+        if type == 'optional':
+            profile = createContentInContainer(survey, "euphorie.module",
+                                            title=unicode(node.title))
+            profile.optional = True
+        else:
+            profile = createContentInContainer(survey,
+                    "euphorie.profilequestion", title=unicode(node.title))
         profile.description = el_unicode(node, 'description')
         profile.question = unicode(node.question)
         profile.external_id = attr_unicode(node, "external-id")
+        for fname in ProfileQuestionLocationFields:
+            setattr(
+                profile, fname, el_unicode(node, fname.replace('_', '-')))
+        profile.use_location_question = el_bool(
+            node, "use-location-question", True)
 
         for child in node.iterchildren(tag=XMLNS + "risk"):
             self.ImportRisk(child, profile)
 
         for child in node.iterchildren(tag=XMLNS + "module"):
             self.ImportModule(child, profile)
+        return profile
 
     def ImportSurvey(self, node, group, version_title):
+        """
+        Create a new :obj:`euphorie.content.survey` object for a
+        :obj:`euphorie.content.surveygroup` given the details for a Survey as an
+        XML node.
+
+        :returns: :obj:`euphorie.content.survey`.
+        """
         survey = createContentInContainer(group, "euphorie.survey",
                                         title=version_title)
-        survey.classification_code = el_string(node, "classification-code")
+        survey.introduction = el_unicode(node, 'introduction')
+        survey.classification_code = el_unicode(node, "classification-code")
         survey.language = el_string(node, "language")
+        tti = getUtility(IToolTypesInfo)
+        survey.tool_type = el_string(node, "tool_type", tti.default_tool_type)
         survey.evaluation_optional = el_bool(node, "evaluation-optional")
         survey.external_id = attr_unicode(node, "external-id")
-
+        if IToolCategory.providedBy(survey):
+            IToolCategory(survey).tool_category = ([
+                x.replace(COMMA_REPLACEMENT, ",").strip()
+                for x in el_unicode(node, 'tool-category', '').split(",")
+            ])
         for child in node.iterchildren():
             if child.tag == XMLNS + "profile-question":
                 self.ImportProfileQuestion(child, survey)
             elif child.tag == XMLNS + "module":
                 self.ImportModule(child, survey)
-
         return survey
 
     def __call__(self, input, surveygroup_title, survey_title):
@@ -275,6 +363,9 @@ class SurveyImporter(object):
 
 
 class SectorImporter(SurveyImporter):
+    """ :returns: :obj:`euphorie.content.survey` root
+    """
+
     def __call__(self, input, sector_title, sector_login, surveygroup_title,
             survey_title):
         if isinstance(input, basestring):
@@ -313,16 +404,27 @@ class SectorImporter(SurveyImporter):
         return root
 
 
-class ImportSurvey(form.Form):
-    fields = field.Fields(IImportSurvey)
+class ImportSurvey(form.SchemaForm):
+    """ The upload view for a :obj:`euphorie.content.sector`
+
+    View name: @@upload
+    """
+    grok.context(ISector)
+    grok.require("euphorie.content.AddNewRIEContent")
+    grok.name("upload")
+    form.wrap(True)
+
+    schema = IImportSurvey
     ignoreContext = True
-    form_name = _(u"Import survey version")
+    form_name = _(u"Import OiRA Tool version")
+
+    importer_factory = SurveyImporter
 
     @button.buttonAndHandler(_(u"Upload"))
     def handleUpload(self, action):
         (data, errors) = self.extractData()
         input = data["file"].data
-        importer = SurveyImporter(self.context)
+        importer = self.importer_factory(self.context)
         try:
             survey = importer(input,
                     data["surveygroup_title"], data["survey_title"])
@@ -334,27 +436,35 @@ class ImportSurvey(form.Form):
 
         IStatusMessage(self.request).addStatusMessage(
             _("upload_success",
-                default=u"Succesfully imported the survey"), type="success")
+                default=u"Succesfully imported the OiRA Tool"), type="success")
         state = getMultiAdapter((survey, self.request),
                 name="plone_context_state")
         self.request.response.redirect(state.view_url())
 
 
-ImportSurveyView = wrap_form(ImportSurvey)
+class ImportSector(form.SchemaForm):
+    """ The upload view for a :obj:`euphorie.content.country`
 
+    View name: @@upload
+    """
+    grok.context(ICountry)
+    grok.require("euphorie.content.ManageCountry")
+    grok.name("upload")
+    form.wrap(True)
 
-class ImportSector(form.Form):
-    fields = field.Fields(IImportSector)
+    schema = IImportSector
     ignoreContext = True
     label = _("title_import_sector_survey",
-            default=u"Import sector and survey")
+            default=u"Import sector and OiRA Tool")
+
+    importer_factory = SectorImporter
 
     @button.buttonAndHandler(_("button_upload", default=u"Upload"))
     def handleUpload(self, action):
         (data, errors) = self.extractData()
         input = data["file"].data
 
-        importer = SectorImporter(self.context)
+        importer = self.importer_factory(self.context)
 
         try:
             sector = importer(input, data["sector_title"],
@@ -368,11 +478,8 @@ class ImportSector(form.Form):
 
         IStatusMessage(self.request).addStatusMessage(
                 _("upload_success",
-                    default=u"Succesfully imported the survey"),
+                    default=u"Succesfully imported the OiRA Tool"),
                 type="success")
         state = getMultiAdapter((sector, self.request),
                 name="plone_context_state")
         self.request.response.redirect(state.view_url())
-
-
-ImportSectorView = wrap_form(ImportSector)
